package djmil.cordacheckers.checkers;

import java.util.HashSet;
import java.util.Set;

import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.annotations.CordaSerializable;

@CordaSerializable
public class Stone {
    
    @CordaSerializable
    public enum Type {
        MAN,
        KING;
    }

    @CordaSerializable
    public enum Color {
        WHITE,
        BLACK;

        public Color opposite() {
            switch (this) {
                case WHITE:
                    return BLACK;
                case BLACK:
                    return WHITE;
            }
            throw new RuntimeException("Unknown Color");
        }
    }

    private final Color color;
    private final Type type;

    @ConstructorForDeserialization
    public Stone(Color color, Type type) {
        this.color = color;
        this.type = type;
    }

    public Color getColor() {
        return color;
    }

    public Type getType() {
        return type;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Stone other = (Stone) obj;
        if (color != other.color)
            return false;
        if (type != other.type)
            return false;
        return true;
    }

    public Set<Move> getJumps(Integer from) {
        Set<Integer> jumps;

        if (type == Type.KING)
            jumps = Move.getJumps(from);
        else if (color == Color.BLACK)
            jumps = Move.getBlackJumps(from);
        else
            jumps = Move.getWhiteJumps(from);

        Set<Move> res = new HashSet<>();

        for (Integer jump : jumps)
            res.add(new Move(from, jump));

        return res;
    }

    public Set<Move> getMoves(Integer from) {
        Set<Integer> steps;
    
        if (type == Type.KING)
            steps = Move.getSteps(from);
        else if (color == Color.BLACK)
            steps = Move.getBlackSteps(from);
        else
            steps = Move.getWhiteSteps(from);

        Set<Move> moves = getJumps(from);       // <<--- Steps + Jumps
        
        for (Integer step : steps) {
            moves.add(new Move(from, step));            
        }

        return moves;
    }

    public Stone promoteIfPossible(Integer to) {
        if ((color == Color.WHITE && to >=  1 && to <=  4) ||
            (color == Color.BLACK && to >= 29 && to <= 32))
            return new Stone(color, Type.KING);
        else 
            return this; 
    }

}
