package djmil.cordacheckers.contracts;

import static djmil.cordacheckers.contracts.GameCommand.requireThat;
import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleCommand;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.checkers.Stone;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;

public class GameBoardContract implements net.corda.v5.ledger.utxo.Contract {
    
    private final static Logger log = LoggerFactory.getLogger(GameBoardContract.class);

    @Override
    public void verify(UtxoLedgerTransaction trx) {
        log.info("GameBoardContract.verify() called");

        requireThat(trx.getCommands().size() == 1, GameCommand.REQUIRE_SINGLE_COMMAND);
        final GameCommand command = getSingleCommand(trx, GameCommand.class);

        switch (command.getAction()) {
            case GAME_PROPOSAL_ACCEPT:
                command.validateGameProposalAccept(trx);
                break;
        
            case GAME_BOARD_MOVE:
                command.validateGameBoardMove(trx, command.getMove());
                break;

            case GAME_BOARD_SURRENDER:
                command.validateGameBoardSurrender(trx);
                break;

            default:
                throw new GameCommand.ActionException();
        }
    }

    public final static Map<Integer, Stone> initialBoard =  Map.ofEntries(
        // Inspired by Checkers notation rules: https://www.bobnewell.net/nucleus/checkers.php
        Map.entry( 1, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),
        Map.entry( 2, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),
        Map.entry( 3, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),
        Map.entry( 4, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),
        Map.entry( 5, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),
        Map.entry( 6, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),
        Map.entry( 7, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),
        Map.entry( 8, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),
        Map.entry( 9, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),
        Map.entry(10, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),
        Map.entry(11, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),
        Map.entry(12, new Stone(Stone.Color.BLACK, Stone.Type.MAN)),

        Map.entry(21, new Stone(Stone.Color.WHITE, Stone.Type.MAN)),
        Map.entry(22, new Stone(Stone.Color.WHITE, Stone.Type.MAN)),
        Map.entry(23, new Stone(Stone.Color.WHITE, Stone.Type.MAN)),
        Map.entry(24, new Stone(Stone.Color.WHITE, Stone.Type.MAN)),
        Map.entry(25, new Stone(Stone.Color.WHITE, Stone.Type.MAN)),
        Map.entry(26, new Stone(Stone.Color.WHITE, Stone.Type.MAN)),
        Map.entry(27, new Stone(Stone.Color.WHITE, Stone.Type.MAN)),
        Map.entry(28, new Stone(Stone.Color.WHITE, Stone.Type.MAN)),
        Map.entry(29, new Stone(Stone.Color.WHITE, Stone.Type.MAN)),
        Map.entry(30, new Stone(Stone.Color.WHITE, Stone.Type.MAN)),
        Map.entry(31, new Stone(Stone.Color.WHITE, Stone.Type.MAN)),
        Map.entry(32, new Stone(Stone.Color.WHITE, Stone.Type.MAN))
    );

}
