package djmil.cordacheckers.contracts;

import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleCommand;
import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleInputState;
import static djmil.cordacheckers.contracts.UtxoLedgerTransactionUtil.getSingleOutputState;

import djmil.cordacheckers.contracts.GameCommand.ActionException;
import djmil.cordacheckers.states.GameBoardState;
import djmil.cordacheckers.states.GameProposalState;
import djmil.cordacheckers.states.GameResultState;
import djmil.cordacheckers.states.GameState;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;

public class GameInfo {

    public final GameCommand command;
    public final MemberX500Name issuer; // a person who has created/updated state (important for CommitSubFlowResponder)
    public final MemberX500Name actor;  // a person who is expected to act on a state (inmportnat for ViewBuilder)
    public final GameState state;   // lates, most actual state

    public GameInfo(UtxoLedgerTransaction utxoTrx) {
        this.command = getSingleCommand(utxoTrx, GameCommand.class);

        switch (this.command.getAction()) {
            case GAME_PROPOSAL_CREATE:
                this.state  = getSingleOutputState(utxoTrx, GameProposalState.class);
                this.issuer = ((GameProposalState)this.state).getIssuerName();
                this.actor  = ((GameProposalState)this.state).getAcquierName();
                return;
            
            case GAME_PROPOSAL_CANCEL:
                this.state  = getSingleInputState(utxoTrx, GameProposalState.class);
                this.issuer = ((GameProposalState)this.state).getIssuerName();
                this.actor  = null;
                return;
            
            case GAME_PROPOSAL_REJECT:
                this.state  = getSingleInputState(utxoTrx, GameProposalState.class);
                this.issuer = ((GameProposalState)this.state).getAcquierName();
                this.actor  = null;
                return;

            case GAME_PROPOSAL_ACCEPT:
                this.state  = getSingleOutputState(utxoTrx, GameBoardState.class);
                this.issuer = getSingleInputState(utxoTrx, GameProposalState.class).getAcquierName();
                this.actor  = ((GameBoardState)this.state).getActivePlayerName();
                return;

            case GAME_BOARD_MOVE:
                this.state  = getSingleOutputState(utxoTrx, GameBoardState.class);
                this.issuer = getSingleInputState(utxoTrx, GameBoardState.class).getActivePlayerName();
                this.actor  = ((GameBoardState)this.state).getActivePlayerName();
                return;
            
            case GAME_BOARD_VICTORY:
                this.state  = getSingleOutputState(utxoTrx, GameResultState.class);
                this.issuer = ((GameResultState)this.state).getWinnerName();
                this.actor  = null;
                return;

            case GAME_BOARD_SURRENDER:
                this.state  = getSingleOutputState(utxoTrx, GameResultState.class);
                this.issuer = ((GameResultState)this.state).getLooserName();
                this.actor  = null;
                return;
        }

        throw new ActionException();
    }

    public boolean isMyAction(MemberX500Name myName) {
        if (this.actor == null)
            return false;
        
        return this.actor.compareTo(myName) == 0;
    }

}
