package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import djmil.cordacheckers.checkers.Move;
import djmil.cordacheckers.checkers.Stone;
import djmil.cordacheckers.contracts.GameBoardContract;
import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.BelongsToContract;

@BelongsToContract(GameBoardContract.class)
public class GameBoardState extends GameState {
    private final Stone.Color activeColor;
    private final Integer moveNumber;
    private final Map<Integer, Stone> board;

    public GameBoardState(GameProposalState gameProposalState) {
        super(gameProposalState.whitePlayer, gameProposalState.blackPlayer, 
            gameProposalState.gameUuid, gameProposalState.message, gameProposalState.participants);
        
        this.board = new LinkedHashMap<Integer, Stone>(GameBoardContract.initialBoard);
        this.activeColor = Stone.Color.WHITE;
        this.moveNumber = 0;
    }

    public GameBoardState(GameBoardState currentGameBoardState, List<Integer> move, String message) {
        super(currentGameBoardState.whitePlayer, currentGameBoardState.blackPlayer, currentGameBoardState.gameUuid, 
            message, currentGameBoardState.participants);

        this.board = new LinkedHashMap<Integer, Stone>(currentGameBoardState.getBoard());
        this.activeColor = Move.apply(move, this.board, currentGameBoardState.getActiveColor());

        this.moveNumber = (currentGameBoardState.activeColor == this.activeColor)
            ? currentGameBoardState.getMoveNumber() // current player has not finished his move jet
            : currentGameBoardState.getMoveNumber() +1;
    }
 
    @ConstructorForDeserialization
    public GameBoardState(MemberX500Name whitePlayer, MemberX500Name blackPlayer, 
            Stone.Color activeColor, Integer moveNumber, Map<Integer, Stone> board, String message, 
            UUID gameUuid, List<PublicKey> participants) {
        super(whitePlayer, blackPlayer, gameUuid, message, participants);

        this.activeColor = activeColor;
        this.moveNumber = moveNumber;
        this.board = board;
    }

    public Stone.Color getActiveColor() {
        return activeColor;
    }

    public Integer getMoveNumber() {
        return moveNumber;
    }

    public MemberX500Name getActivePlayerName() {
        return activeColor == Stone.Color.WHITE ? whitePlayer : blackPlayer;
    }

    public MemberX500Name getIdelPlayerName() {
        return activeColor == Stone.Color.WHITE ? blackPlayer : whitePlayer;
    }

    public Map<Integer, Stone> getBoard() {
        return board;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        GameBoardState other = (GameBoardState) obj;
        if (activeColor != other.activeColor)
            return false;
        if (moveNumber == null) {
            if (other.moveNumber != null)
                return false;
        } else if (!moveNumber.equals(other.moveNumber))
            return false;
        if (board == null) {
            if (other.board != null)
                return false;
        } else if (!board.equals(other.board))
            return false;
        return true;
    }

}
