package djmil.cordacheckers.user;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.springframework.stereotype.Service;

import djmil.cordacheckers.cordaclient.CordaClient;
import djmil.cordacheckers.cordaclient.dao.HoldingIdentity;
import djmil.cordacheckers.cordaclient.dao.VirtualNode;

@Service
public class HoldingIdentityResolver {
    static final Locale locale = Locale.getDefault();

    /*
     * NOTE: Proof-of-Concept impementation 
     *  Ideally, we want to be able to force update cache, had the corda
     *  cluster configuration changed (aka in case of a cache miss)
     */ 
    Map<String, HoldingIdentity> player; // PlayerName to ShortHash
    HoldingIdentity custodian;

    HoldingIdentityResolver(CordaClient cordaClient) {
        this.player = new HashMap<>();

        fetchVNodeList(cordaClient);
    }

    private void fetchVNodeList(CordaClient cordaClient) {
        final List<VirtualNode> vnodeList = cordaClient.fetchVirtualNodeList();

        for (VirtualNode vNode : vnodeList) {
            final var identity = vNode.holdingIdentity();
            try {            
                if (!identity.isCordaCheckers())
                    continue;

                if (identity.isPlayer())
                    player.put(identity.getName().toLowerCase(locale), identity);

                if (identity.isCustodian())
                    custodian = identity; // <<-- there shall be only one custodian
            }
            catch (Exception e) {
                System.err.println("Incomprehensible VirtualNode identity " +identity);
            } 
        }
    }

    /*
     * @param playerName 
     *  HoldingIdentity x500 name typically looks like 
     *  "CN=Bob, OU=Player, O=Checkers, L=Kyiv, C=UA"
     *  CN - is a common name, expected to be unique for CordaCheckers setup.
     * 
     * @return HoldingIdentity
     */
    public HoldingIdentity getByUsername(String userName) {
        return this.player.get(userName.toLowerCase(locale));
    }

    public HoldingIdentity getCustodian() {
        return this.custodian;
    }
}
